package com.googlecode.marrowboy;

import java.io.PrintStream;
import java.util.ArrayList;
import java.util.List;

import org.hamcrest.Description;
import org.hamcrest.Matcher;
import org.hamcrest.MatcherAssert;
import org.hamcrest.StringDescription;

import com.googlecode.marrowboy.assertionresult.AssertionResult;
import com.googlecode.marrowboy.assertionresult.TestRunResult;
import com.googlecode.marrowboy.writers.OutputConfiguration;
import com.googlecode.marrowboy.writers.ReportWriter;

/**
 * Provides methods which can be used in place of the Hamcrest methods, and
 * methods to configure the behaviour of MarrowBoy reports.
 * 
 * This class contains the only methods which are expected to be called by
 * users. It is expected to be used statically so instantiation is forbidden.
 * 
 * @author mjg
 */
public final class MarrowBoyAssert {

	private static final TestRunResult messages = new TestRunResult();

	private static final String UNSPECIFIED_REASON = "";

	private static List<OutputConfiguration> outputConfigurations = new ArrayList<OutputConfiguration>();

	static {

		Runtime.getRuntime().addShutdownHook(new Thread() {

			@Override
			public void run() {

				for (OutputConfiguration thisConfig : outputConfigurations) {

					if (thisConfig.getReportWriter() != null && thisConfig.getPrintStream() != null) {
						thisConfig.getReportWriter().writeReport(messages, thisConfig.getPrintStream());
					}

				}

			}

		});

	}

	/**
	 * No instantiation.
	 */
	private MarrowBoyAssert() {
	}

	/**
	 * Allows users to specify the {@link ReportWriter} to use (eg
	 * {@link PlainTextWriter}), and the {@link PrintStream} to write to.
	 * 
	 * If either of these is null, the report will not be written anywhere. If
	 * this method is called more than once, the last call will overwrite all
	 * previous calls. If it is never called, we use a {@link PlainTextWriter}
	 * to {@link System#out}.
	 * 
	 * @param reportWriter
	 *            the {@link ReportWriter} to use.
	 * 
	 * @param printStream
	 *            the {@link PrintStream} to write to. Easy options for this
	 *            argument are {@link System#out} or
	 *            {@link PrintStream#PrintStream(String)}
	 */
	public static void addReportWriter(ReportWriter reportWriter, PrintStream printStream) {
		outputConfigurations.add(new OutputConfiguration(reportWriter, printStream));
	}

	/**
	 * Remove all {@link OutputConfiguration}s - maybe useful because the
	 * default is to include {@link PlainTextWriter} to {@link System#out}
	 */
	public static void clearReportWriters() {
		outputConfigurations.clear();
	}

	/**
	 * Provides the same functionality as
	 * {@link MatcherAssert#assertThat(Object, Matcher)}, except that it also
	 * adds the assertion to the generated report.
	 * 
	 * @param actual
	 *            the object produced by the system under test.
	 * @param matcher
	 *            the {@link Matcher} which will be applied to the first
	 *            argument.
	 * @return the {@link AssertionResult} which was generated for this
	 *         assertion.
	 */
	public static <T> AssertionResult assertThat(T actual, Matcher<? super T> matcher) {

		try {
			MatcherAssert.assertThat(actual, matcher);

		} catch (AssertionError ae) {
			realLogThat(UNSPECIFIED_REASON, actual, matcher, false);
			throw ae;

		}

		return realLogThat(UNSPECIFIED_REASON, actual, matcher, true);

	}

	/**
	 * Provides the same functionality as
	 * {@link MatcherAssert#assertThat(String, Object, Matcher)}, except that it
	 * also adds the assertion to the generated report.
	 * 
	 * @deprecated We prefer the idiom
	 *             <code>assertThat(actual, matcher).because(reason)</code>
	 * 
	 * @param reason
	 *            some textual description of the assertion.
	 * @param actual
	 *            the object produced by the system under test.
	 * @param matcher
	 *            the {@link Matcher} which will be applied to the first
	 *            argument.
	 * @return the {@link AssertionResult} which was generated for this
	 *         assertion.
	 */
	@Deprecated
	public static <T> AssertionResult assertThat(String reason, T actual, Matcher<? super T> matcher) {

		try {
			MatcherAssert.assertThat(actual, matcher);

		} catch (AssertionError ae) {
			realLogThat(reason, actual, matcher, false);
			throw ae;

		}

		return realLogThat(reason, actual, matcher, true);

	}

	private static <T> AssertionResult realLogThat(String reason, T actual, Matcher<? super T> matcher, boolean wasSuccessful) {

		// if we pop this many elements off the call stack, we'll be looking at
		// the method which first called us.
		int stackPopAmount = 3;

		StackTraceElement stackTraceElement = Thread.currentThread().getStackTrace()[stackPopAmount];

		String actualString = actual.toString();

		// TODO: try to do other types nicely here

		Description description = new StringDescription();
		matcher.describeTo(description);

		return messages.logMessageFromStackTraceElement(stackTraceElement, reason, actualString, description.toString(), wasSuccessful);

	}
}
