package com.googlecode.marrowboy.assertionresult;

import java.util.Collection;
import java.util.LinkedHashMap;
import java.util.Map;

/**
 * Encapsulates a single test class, (possibly) containing multiple test
 * methods.
 * 
 * @author mjg
 */
public class ClassResult {

	private final String name;
	private final Map<String, MethodResult> methodResults;

	/**
	 * Constructor.
	 * 
	 * @param className
	 *            the name of the class
	 */
	public ClassResult(String className) {
		methodResults = new LinkedHashMap<String, MethodResult>();
		this.name = className;
	}

	/**
	 * Getter
	 * 
	 * @return the class name
	 */
	public String getName() {
		return name;
	}

	/**
	 * Get a {@link MethodResult} by name
	 * 
	 * @param methodName
	 *            the name of the method
	 * @return the MethodResult object for the given name. Will create a new
	 *         {@link MethodResult} rather than return null.
	 */
	public MethodResult getMethodResult(String methodName) {

		if (methodResults.get(methodName) == null) {
			methodResults.put(methodName, new MethodResult(methodName));
		}

		return methodResults.get(methodName);
	}

	/**
	 * How many methods passed in this class?
	 * 
	 * @return the number of methods which passed
	 */
	public int getPassCount() {
		int count = 0;
		for (MethodResult methodResult : methodResults.values()) {
			if (methodResult.allPassed()) {
				count++;
			}
		}
		return count;
	}

	/**
	 * How many methods failed in this class?
	 * 
	 * @return the number of assertions which methods
	 */
	public int getFailCount() {
		int count = 0;
		for (MethodResult methodResult : methodResults.values()) {
			if (!methodResult.allPassed()) {
				count++;
			}
		}
		return count;
	}

	/**
	 * How many methods in this class?
	 * 
	 * @return the number of methods
	 */
	public int getTotalCount() {
		return methodResults.size();
	}

	/**
	 * Gets all the {@link MethodResult}s in this {@link ClassResult}.
	 * 
	 * @return a <code>Collection&lt;MethodResult&gt;</code> of all method
	 *         results in this class
	 */
	public Collection<MethodResult> getMethodResults() {
		return methodResults.values();
	}

	/**
	 * Convenience method to check if all methods passed
	 * 
	 * @return whether all the methods passed
	 */
	public boolean allPassed() {
		return (getFailCount() == 0);
	}

}
